
////////////////////////////////////////////////////////////
// Headers
////////////////////////////////////////////////////////////
#include <SFML/Graphics.hpp>
#include <GL/gl.h>
#include <GL/glu.h>


////////////////////////////////////////////////////////////
/// Entry point of application
///
/// \return Application exit code
///
////////////////////////////////////////////////////////////
int main()
{
    // Create main window
    sfRenderWindow App(sfVideoMode(800, 600), "SFML OpenGL", false);

    // Create a sprite for the background
    sfImage BackgroundImage;
    if (!BackgroundImage.LoadFromFile("datas/opengl/background.jpg"))
        return EXIT_FAILURE;
    sfSprite Background(BackgroundImage);

    // Load an OpenGL texture.
    // We could directly use a sfImage as an OpenGL texture (with its Bind() member function),
    // but here we want more control on it (generate mipmaps, ...) so we create a new one
    GLuint Texture = 0;
    {
        sfImage Image;
        if (!Image.LoadFromFile("datas/opengl/texture.jpg"))
            return EXIT_FAILURE;
        glGenTextures(1, &Texture);
        glBindTexture(GL_TEXTURE_2D, Texture);
        gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, Image.GetWidth(), Image.GetHeight(), GL_RGBA, GL_UNSIGNED_BYTE, Image.GetPixelsPtr());
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
    }

    // Create a clock for measuring the time elapsed
    sfClock Clock;

    // Start game loop
    bool Running = true;
    while (Running)
    {
        // Process events
        sfEvent Event;
        while (App.GetEvent(Event))
        {
            // Close window : exit
            if (Event.Type == sfEvent::Close)
                Running = false;

            // Escape key : exit
            if ((Event.Type == sfEvent::KeyPressed) && (Event.Key.Code == sfKey::Escape))
                Running = false;
        }

        // Draw background
        App.Draw(Background);

        // Begin OpenGL rendering
        App.BeginOpenGL();

            // Clear depth buffer
            glClearDepth(1.f);
            glClear(GL_DEPTH_BUFFER_BIT);

            // Enable Z-buffer read and write
            glEnable(GL_DEPTH_TEST);
            glDepthMask(GL_TRUE);

            // Setup a perspective projection
            glMatrixMode(GL_PROJECTION);
            glLoadIdentity();
            gluPerspective(90.f, 1.f, 1.f, 500.f);

            // Apply some transformations
            glMatrixMode(GL_MODELVIEW);
            glLoadIdentity();
            glTranslatef(0.f, 0.f, -200.f);
            glRotatef(Clock.GetElapsedTime() * 50, 1.f, 0.f, 0.f);
            glRotatef(Clock.GetElapsedTime() * 30, 0.f, 1.f, 0.f);
            glRotatef(Clock.GetElapsedTime() * 90, 0.f, 0.f, 1.f);

            // Bind our texture
            glBindTexture(GL_TEXTURE_2D, Texture);

            // Draw a cube
            glBegin(GL_QUADS);

                glTexCoord2f(0, 0); glVertex3f(-50.f, -50.f, -50.f);
                glTexCoord2f(0, 1); glVertex3f(-50.f,  50.f, -50.f);
                glTexCoord2f(1, 1); glVertex3f( 50.f,  50.f, -50.f);
                glTexCoord2f(1, 0); glVertex3f( 50.f, -50.f, -50.f);

                glTexCoord2f(0, 0); glVertex3f(-50.f, -50.f, 50.f);
                glTexCoord2f(0, 1); glVertex3f(-50.f,  50.f, 50.f);
                glTexCoord2f(1, 1); glVertex3f( 50.f,  50.f, 50.f);
                glTexCoord2f(1, 0); glVertex3f( 50.f, -50.f, 50.f);

                glTexCoord2f(0, 0); glVertex3f(-50.f, -50.f, -50.f);
                glTexCoord2f(0, 1); glVertex3f(-50.f,  50.f, -50.f);
                glTexCoord2f(1, 1); glVertex3f(-50.f,  50.f,  50.f);
                glTexCoord2f(1, 0); glVertex3f(-50.f, -50.f,  50.f);

                glTexCoord2f(0, 0); glVertex3f(50.f, -50.f, -50.f);
                glTexCoord2f(0, 1); glVertex3f(50.f,  50.f, -50.f);
                glTexCoord2f(1, 1); glVertex3f(50.f,  50.f,  50.f);
                glTexCoord2f(1, 0); glVertex3f(50.f, -50.f,  50.f);

                glTexCoord2f(0, 1); glVertex3f(-50.f, -50.f,  50.f);
                glTexCoord2f(0, 0); glVertex3f(-50.f, -50.f, -50.f);
                glTexCoord2f(1, 0); glVertex3f( 50.f, -50.f, -50.f);
                glTexCoord2f(1, 1); glVertex3f( 50.f, -50.f,  50.f);

                glTexCoord2f(0, 1); glVertex3f(-50.f, 50.f,  50.f);
                glTexCoord2f(0, 0); glVertex3f(-50.f, 50.f, -50.f);
                glTexCoord2f(1, 0); glVertex3f( 50.f, 50.f, -50.f);
                glTexCoord2f(1, 1); glVertex3f( 50.f, 50.f,  50.f);

            glEnd();

        // End OpenGL rendering
        App.EndOpenGL();

        // Draw some text on top of our OpenGL object
        sfString Text("This is a rotating cube");
        Text.SetLeft(230.f);
        Text.SetTop(300.f);
        Text.SetColor(sfColor(128, 0, 128));
        App.Draw(Text);

        // Finally, display rendered frame on screen
        App.Display();
    }

    // Don't forget to destroy our texture
    glDeleteTextures(1, &Texture);

    return EXIT_SUCCESS;
}
