
////////////////////////////////////////////////////////////
// Headers
////////////////////////////////////////////////////////////
#include <SFML/Graphics.hpp>
#include <map>


////////////////////////////////////////////////////////////
/// Entry point of application
///
/// \return Application exit code
///
////////////////////////////////////////////////////////////
int main()
{
    // Check that the system can use post effects
    if (sfPostFX::CanUsePostFX() == false)
        return EXIT_SUCCESS;

    // Create main window
    sfRenderWindow App(sfVideoMode(800, 600), "SFML PostFX", false);

    // Load a cute background image to display :)
    sfImage BackgroundImage;
    if (!BackgroundImage.LoadFromFile("datas/post-fx/background.jpg"))
        return EXIT_FAILURE;
    sfSprite Background(BackgroundImage);

    // Load the image needed for the wave effect
    sfImage WaveImage;
    if (!WaveImage.LoadFromFile("datas/post-fx/wave.jpg"))
        return EXIT_FAILURE;

    // Load all effects
    std::map<std::string, sfPostFX> Effects;
    Effects["nothing"].LoadFromFile("datas/post-fx/nothing.sfx");
    Effects["blur"].LoadFromFile("datas/post-fx/blur.sfx");
    Effects["colorize"].LoadFromFile("datas/post-fx/colorize.sfx");
    Effects["fisheye"].LoadFromFile("datas/post-fx/fisheye.sfx");
    Effects["wave"].LoadFromFile("datas/post-fx/wave.sfx");
    std::map<std::string, sfPostFX>::iterator CurrentEffect = Effects.find("nothing");

    // Do specific initializations
    Effects["nothing"].SetTexture("framebuffer", NULL);
    Effects["blur"].SetTexture("framebuffer", NULL);
    Effects["blur"].SetParameter("offset", 0.f);
    Effects["colorize"].SetTexture("framebuffer", NULL);
    Effects["colorize"].SetParameter("color", 1.f, 1.f, 1.f);
    Effects["fisheye"].SetTexture("framebuffer", NULL);
    Effects["wave"].SetTexture("framebuffer", NULL);
    Effects["wave"].SetTexture("wave", &WaveImage);

    // Define a string for displaying current effect description
    sfString CurFXStr;
    CurFXStr.SetText("Current effect is \"" + CurrentEffect->first + "\"");
    CurFXStr.SetFont("datas/post-fx/cheeseburger.ttf");
    CurFXStr.SetLeft(20.f);
    CurFXStr.SetTop(0.f);

    // Define a string for displaying help
    sfString InfoStr;
    InfoStr.SetText("Move your mouse to change effect parameters\nPress numpad + and - to change effect");
    InfoStr.SetFont("datas/post-fx/cheeseburger.ttf");
    InfoStr.SetLeft(20.f);
    InfoStr.SetTop(520.f);
    InfoStr.SetColor(sfColor(200, 100, 150));

    // Start game loop
    bool Running = true;
    while (Running)
    {
        // Process events
        sfEvent Event;
        while (App.GetEvent(Event))
        {
            // Close window : exit
            if (Event.Type == sfEvent::Close)
                Running = false;

            if (Event.Type == sfEvent::KeyPressed)
            {
                // Escape key : exit
                if (Event.Key.Code == sfKey::Escape)
                    Running = false;

                // Add key : next effect
                if (Event.Key.Code == sfKey::Add)
                {
                    CurrentEffect++;
                    if (CurrentEffect == Effects.end())
                        CurrentEffect = Effects.begin();
                    CurFXStr.SetText("Current effect is \"" + CurrentEffect->first + "\"");
                }

                // Subtract key : previous effect
                if (Event.Key.Code == sfKey::Subtract)
                {
                    if (CurrentEffect == Effects.begin())
                        CurrentEffect = Effects.end();
                    CurrentEffect--;
                    CurFXStr.SetText("Current effect is \"" + CurrentEffect->first + "\"");
                }
            }
        }

        // Get the mouse position in the range [0, 1]
        float X = App.GetInput().GetMouseX() / static_cast<float>(App.GetWidth());
        float Y = App.GetInput().GetMouseY() / static_cast<float>(App.GetHeight());

        // Update the current effect
        if      (CurrentEffect->first == "blur")     CurrentEffect->second.SetParameter("offset", X * Y * 0.1f);
        else if (CurrentEffect->first == "colorize") CurrentEffect->second.SetParameter("color", 0.3f, X, Y);
        else if (CurrentEffect->first == "fisheye")  CurrentEffect->second.SetParameter("mouse", X, 1.f - Y);
        else if (CurrentEffect->first == "wave")     CurrentEffect->second.SetParameter("offset", -X, Y);

        // Draw background and apply the post-fx
        App.Draw(Background);
        App.Draw(CurrentEffect->second);

        // Draw interface strings
        App.Draw(CurFXStr);
        App.Draw(InfoStr);

        // Finally, display the rendered frame on screen
        App.Display();
    }

    return EXIT_SUCCESS;
}
