////////////////////////////////////////////////////////////
//
// SFML - Simple and Fast Multimedia Library
// Copyright (C) 2007 Laurent Gomila (laurent.gom@gmail.com)
//
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//
////////////////////////////////////////////////////////////

#ifndef SFML_AUDIORESOURCE_HPP
#define SFML_AUDIORESOURCE_HPP

////////////////////////////////////////////////////////////
// Headers
////////////////////////////////////////////////////////////
#include <SFML/Config.hpp>


namespace sf_private
{
////////////////////////////////////////////////////////////
/// Abstract base class for every class that owns a
/// device-dependant resource -- allow them to initialize / shutdown
/// even when audio context is not created
////////////////////////////////////////////////////////////
class SFML_API sfAudioResource
{
public :

    ////////////////////////////////////////////////////////////
    /// Destroy all audio resources that need a valid context
    ///
    ////////////////////////////////////////////////////////////
    virtual void DestroyAudioResources() = 0;

protected :

    ////////////////////////////////////////////////////////////
    /// Default constructor
    ///
    ////////////////////////////////////////////////////////////
    sfAudioResource();

    ////////////////////////////////////////////////////////////
    /// Copy constructor
    ///
    ////////////////////////////////////////////////////////////
    sfAudioResource(const sfAudioResource&);

    ////////////////////////////////////////////////////////////
    /// Destructor
    ///
    ////////////////////////////////////////////////////////////
    virtual ~sfAudioResource();
};

} // namespace sf_private

#endif // SFML_AUDIORESOURCE_HPP
