////////////////////////////////////////////////////////////
//
// SFML - Simple and Fast Multimedia Library
// Copyright (C) 2007 Laurent Gomila (laurent.gom@gmail.com)
//
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// Headers
////////////////////////////////////////////////////////////
#include <SFML/Graphics/Color.hpp>


////////////////////////////////////////////////////////////
// Static member data
////////////////////////////////////////////////////////////
const sfColor sfColor::Black(0, 0, 0);
const sfColor sfColor::White(255, 255, 255);
const sfColor sfColor::Red(255, 0, 0);
const sfColor sfColor::Green(0, 255, 0);
const sfColor sfColor::Blue(0, 0, 255);
const sfColor sfColor::Yellow(255, 255, 0);
const sfColor sfColor::Magenta(255, 0, 255);
const sfColor sfColor::Cyan(0, 255, 255);


////////////////////////////////////////////////////////////
/// Default constructor
////////////////////////////////////////////////////////////
sfColor::sfColor() :
r(0),
g(0),
b(0),
a(255)
{

}


////////////////////////////////////////////////////////////
/// Construct the color from a RGBA 32 bits integer
////////////////////////////////////////////////////////////
sfColor::sfColor(sfUint32 ColorRGBA) :
r(static_cast<sfUint8>((ColorRGBA & 0x000000FF))),
g(static_cast<sfUint8>((ColorRGBA & 0x0000FF00) >> 8)),
b(static_cast<sfUint8>((ColorRGBA & 0x00FF0000) >> 16)),
a(static_cast<sfUint8>((ColorRGBA & 0xFF000000) >> 24))
{

}


////////////////////////////////////////////////////////////
/// Construct the color from its 4 RGBA components
////////////////////////////////////////////////////////////
sfColor::sfColor(sfUint8 R, sfUint8 G, sfUint8 B, sfUint8 A) :
r(R),
g(G),
b(B),
a(A)
{

}


////////////////////////////////////////////////////////////
/// Get the RGBA representation of the color
////////////////////////////////////////////////////////////
sfUint32 sfColor::ToRGBA() const
{
    return (a << 24) | (b << 16) | (g << 8) | r;
}


////////////////////////////////////////////////////////////
/// Compare two colors (for equality)
////////////////////////////////////////////////////////////
bool sfColor::operator ==(const sfColor& Other) const
{
    return (r == Other.r) && (g == Other.g) && (b == Other.b) && (a == Other.a);
}


////////////////////////////////////////////////////////////
/// Compare two colors (for difference)
////////////////////////////////////////////////////////////
bool sfColor::operator !=(const sfColor& Other) const
{
    return (r != Other.r) || (g != Other.g) || (b != Other.b) || (a != Other.a);
}
