////////////////////////////////////////////////////////////
//
// SFML - Simple and Fast Multimedia Library
// Copyright (C) 2007 Laurent Gomila (laurent.gom@gmail.com)
//
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//
////////////////////////////////////////////////////////////

#ifndef SFML_COLOR_HPP
#define SFML_COLOR_HPP

////////////////////////////////////////////////////////////
// Headers
////////////////////////////////////////////////////////////
#include <SFML/Config.hpp>


////////////////////////////////////////////////////////////
/// sfColor is an utility class for manipulating colors
////////////////////////////////////////////////////////////
class SFML_API sfColor
{
public :

    ////////////////////////////////////////////////////////////
    /// Default constructor
    ///
    ////////////////////////////////////////////////////////////
    sfColor();

    ////////////////////////////////////////////////////////////
    /// Construct the color from a RGBA 32 bits integer
    ///
    /// \param ColorRGBA : RGBA color
    ///
    ////////////////////////////////////////////////////////////
    explicit sfColor(sfUint32 ColorRGBA);

    ////////////////////////////////////////////////////////////
    /// Construct the color from its 4 RGBA components
    ///
    /// \param R : Red component   (0 .. 255)
    /// \param G : Green component (0 .. 255)
    /// \param B : Blue component  (0 .. 255)
    /// \param A : Alpha component (0 .. 255) (255 by default)
    ///
    ////////////////////////////////////////////////////////////
    sfColor(sfUint8 R, sfUint8 G, sfUint8 B, sfUint8 A = 255);

    ////////////////////////////////////////////////////////////
    /// Get the RGBA representation of the color
    ///
    /// \return 32 bits integer representing the color in RGBA
    ///
    ////////////////////////////////////////////////////////////
    sfUint32 ToRGBA() const;

    ////////////////////////////////////////////////////////////
    /// Compare two colors (for equality)
    ///
    /// \param Other : Color to compare
    ///
    /// \return True if colors are equal
    ///
    ////////////////////////////////////////////////////////////
    bool operator ==(const sfColor& Other) const;

    ////////////////////////////////////////////////////////////
    /// Compare two colors (for difference)
    ///
    /// \param Other : Color to compare
    ///
    /// \return True if colors are different
    ///
    ////////////////////////////////////////////////////////////
    bool operator !=(const sfColor& Other) const;

    ////////////////////////////////////////////////////////////
    // Static member data
    ////////////////////////////////////////////////////////////
    static const sfColor Black;   ///< Black predefined color
    static const sfColor White;   ///< White predefined color
    static const sfColor Red;     ///< Red predefined color
    static const sfColor Green;   ///< Green predefined color
    static const sfColor Blue;    ///< Blue predefined color
    static const sfColor Yellow;  ///< Yellow predefined color
    static const sfColor Magenta; ///< Magenta predefined color
    static const sfColor Cyan;    ///< Cyan predefined color

    ////////////////////////////////////////////////////////////
    // Member data
    ////////////////////////////////////////////////////////////
    sfUint8 r; ///< Red component
    sfUint8 g; ///< Green component
    sfUint8 b; ///< Blue component
    sfUint8 a; ///< Alpha (transparency) component
};


#endif // SFML_COLOR_HPP
